@@ -21,17 +21,17 @@
 **************************************************************************** */
 #include "kcubeboxwidget.h"
 
-#include <kapplication.h>
-#include <kconfig.h>
-#include <qlayout.h>
-#include <qtimer.h>
+#include <KConfig>
+#include <QTimer>
+#include <QPainter>
+
 #include <assert.h>
 #include <kcursor.h>
 
 #include "prefs.h"
 
-KCubeBoxWidget::KCubeBoxWidget(const int d,QWidget *parent,const char *name)
-        : QWidget(parent,name),
+KCubeBoxWidget::KCubeBoxWidget(const int d,QWidget *parent)
+        : QWidget(parent),
           CubeBoxBase<KCubeWidget>(d)
 {
    init();
@@ -39,38 +39,38 @@ KCubeBoxWidget::KCubeBoxWidget(const int d,QWidget *parent,const char *name)
 
 
 
-KCubeBoxWidget::KCubeBoxWidget(CubeBox& box,QWidget *parent,const char *name)
-      :QWidget(parent,name),
+KCubeBoxWidget::KCubeBoxWidget(CubeBox& box,QWidget *parent)
+      :QWidget(parent),
        CubeBoxBase<KCubeWidget>(box.dim())
-{  
+{
    init();
-   
+
    int i,j;
    for(i=0;i<dim();i++)
       for(j=0;j<dim();j++)
       {
          *cubes[i][j]=*box[i][j];
       }
-      
+
    currentPlayer=(KCubeBoxWidget::Player)box.player();
 }
 
 
 
-KCubeBoxWidget::KCubeBoxWidget(const KCubeBoxWidget& box,QWidget *parent,const char *name)
-      :QWidget(parent,name),
+KCubeBoxWidget::KCubeBoxWidget(const KCubeBoxWidget& box,QWidget *parent)
+      :QWidget(parent),
        CubeBoxBase<KCubeWidget>(box.dim())
-{  
+{
    init();
-   
+
    int i,j;
    for(i=0;i<dim();i++)
       for(j=0;j<dim();j++)
       {
          *cubes[i][j]=*box.cubes[i][j];
       }
-   
-      
+
+
    currentPlayer=box.currentPlayer;
 }
 
@@ -82,20 +82,44 @@ KCubeBoxWidget::~KCubeBoxWidget()
     stopActivities();
    if(cubes)
       deleteCubes();
-   if(undoBox)
-      delete undoBox;
+   delete undoBox;
 }
 
 void KCubeBoxWidget::loadSettings(){
-  setColor(KCubeBoxWidget::One, Prefs::color1());
-  setColor(KCubeBoxWidget::Two, Prefs::color2());
+  bool reColorCubes = ((color1 != Prefs::color1()) ||
+                       (color2 != Prefs::color2()) ||
+                       (color0 != Prefs::color0()));
+  bool reSizeCubes  = (dim() != Prefs::cubeDim());
+  qDebug() << "KCubeBoxWidget::loadSettings reColorCubes:" << reColorCubes
+           << "reSizeCubes:" << reSizeCubes; // IDW
+
+  color1 = Prefs::color1();
+  color2 = Prefs::color2();
+  color0 = Prefs::color0();
+  setDim (Prefs::cubeDim());
+
+  if (reColorCubes) {
+     makeStatusPixmaps (sWidth);		// Make new status pixmaps.
+     emit colorChanged(currentPlayer);		// Change color in status bar.
+  }
+  if (reSizeCubes) {
+     reCalculateGraphics (width(), height());
+  }
+  else if (reColorCubes) {
+     makeSVGCubes (cubeSize);
+     setColors ();
+  }
 
-  setDim(Prefs::cubeDim());
   brain.setSkill( Prefs::skill() );
 
   setComputerplayer(KCubeBoxWidget::One, Prefs::computerPlayer1());
   setComputerplayer(KCubeBoxWidget::Two, Prefs::computerPlayer2());
   checkComputerplayer(currentPlayer);
+
+  if (reSizeCubes) {
+     // Start a new game (in the KJumpingCube object).
+     emit dimensionsChanged();
+  }
 }
 
 KCubeBoxWidget& KCubeBoxWidget::operator=(const KCubeBoxWidget& box)
@@ -106,14 +130,14 @@ KCubeBoxWidget& KCubeBoxWidget::operator=(const KCubeBoxWidget& box)
       {
          setDim(box.dim());
       }
-      
-      
+
+
       for(int i=0;i<dim();i++)
          for(int j=0;j<dim();j++)
          {
             *cubes[i][j]=*box.cubes[i][j];
          }
-         
+
       currentPlayer=box.currentPlayer;
    }
    return *this;
@@ -125,33 +149,33 @@ KCubeBoxWidget& KCubeBoxWidget::operator=(CubeBox& box)
    {
       setDim(box.dim());
    }
-   
+
    for(int i=0;i<dim();i++)
       for(int j=0;j<dim();j++)
       {
          *cubes[i][j]=*box[i][j];
       }
-      
+
    currentPlayer=(KCubeBoxWidget::Player)box.player();
-      
+
    return *this;
 }
 
 void KCubeBoxWidget::reset()
 {
    stopActivities();
-   
+
    int i,j;
    for(i=0;i<dim();i++)
       for(j=0;j<dim();j++)
       {
          cubes[i][j]->reset();
       }
-      
+
    KCubeWidget::enableClicks(true);
 
    currentPlayer=One;
-   
+
    emit playerChanged(One);
    checkComputerplayer(One);
 }
@@ -160,18 +184,18 @@ void KCubeBoxWidget::undo()
 {
    if(isActive())
       return;
-   
+
    Player oldPlayer=currentPlayer;
-   
+
    *this=*undoBox;
-   
+
    if(oldPlayer!=currentPlayer)
       emit playerChanged(currentPlayer);
 
    checkComputerplayer(currentPlayer);
-  
+
 }
-      
+
 void KCubeBoxWidget::getHint()
 {
    if(isActive())
@@ -185,28 +209,25 @@ void KCubeBoxWidget::getHint()
       }
 
    int row=0,column=0;
-   CubeBox field=*this;
+   CubeBox field=CubeBox(*this);
 
    emit startedThinking();
-   bool canceled=!brain.getHint(row,column,(CubeBox::Player)currentPlayer,field);
+   bool done = brain.getHint(row,column,(CubeBox::Player)currentPlayer,field);
    emit stoppedThinking();
 
-   if(canceled)
-   {
-      return;  // return if thinking was stopped
+   if (done) {
+      cubes[row][column]->showHint();
    }
-   cubes[row][column]->showHint();
+   // If (! done), we interrupted the brain, so we do not want the hint.
 }
 
-void KCubeBoxWidget::setColor(Player player,QPalette color)
+void KCubeBoxWidget::setColors ()
 {
-   KCubeWidget::setColor((Cube::Owner)player,color);
-
-   for(int row=0;row<dim();row++)
-      for(int col=0;col<dim();col++)
-      {
+   for (int row=0; row<dim(); row++) {
+      for (int col=0; col<dim(); col++) {
          cubes[row][col]->updateColors();
       }
+   }
 }
 
 void KCubeBoxWidget::setDim(int d)
@@ -226,7 +247,7 @@ void KCubeBoxWidget::setComputerplayer(Player player,bool flag)
       computerPlTwo=flag;
 }
 
-  
+
 void KCubeBoxWidget::stopActivities()
 {
    if(moveTimer->isActive())
@@ -242,7 +263,7 @@ void KCubeBoxWidget::stopActivities()
 
 }
 
-void KCubeBoxWidget::saveProperties(KConfigBase* config)
+void KCubeBoxWidget::saveProperties(KConfigGroup& config)
 {
    if(isMoving())
    {
@@ -253,10 +274,10 @@ void KCubeBoxWidget::saveProperties(KConfigBase* config)
       stopActivities();
 
    // save current player
-   config->writeEntry("onTurn",(int)currentPlayer);
+   config.writeEntry("onTurn",(int)currentPlayer);
 
-   QStrList list;
-   list.setAutoDelete(true);
+   QStringList list;
+   //list.setAutoDelete(true);
    QString owner, value, key;
    int cubeDim=dim();
 
@@ -266,39 +287,39 @@ void KCubeBoxWidget::saveProperties(KConfigBase* config)
 	key.sprintf("%u,%u",row,column);
 	owner.sprintf("%u",cubes[row][column]->owner());
 	value.sprintf("%u",cubes[row][column]->value());
-	list.append(owner.ascii());
-	list.append(value.ascii());
-	config->writeEntry(key , list);
+	list.append(owner.toAscii());
+	list.append(value.toAscii());
+	config.writeEntry(key,list);
 
 	list.clear();
       }
-  config->writeEntry("CubeDim",dim());
+  config.writeEntry("CubeDim",dim());
 }
 
-void KCubeBoxWidget::readProperties(KConfigBase* config)
+void KCubeBoxWidget::readProperties(const KConfigGroup& config)
 {
-  QStrList list;
-  list.setAutoDelete(true);
+  QStringList list;
+  //list.setAutoDelete(true);
   QString owner, value, key;
-  setDim(config->readNumEntry("CubeDim",5));
+  setDim(config.readEntry("CubeDim",5));
   int cubeDim=dim();
 
   for(int row=0; row < cubeDim ; row++)
     for(int column=0; column < cubeDim ; column++)
       {
 	key.sprintf("%u,%u",row,column);
-	config->readListEntry(key, list);
-	owner=list.first();
-	value=list.next();
+	list = config.readEntry(key,QStringList());
+	owner=list.at(0);
+	value=list.at(1);
 	cubes[row][column]->setOwner((KCubeWidget::Owner)owner.toInt());
 	cubes[row][column]->setValue(value.toInt());
-	
+
 	list.clear();
       }
 
 
    // set current player
-   int onTurn=config->readNumEntry("onTurn",1);
+   int onTurn=config.readEntry("onTurn",1);
    currentPlayer=(Player)onTurn;
    emit playerChanged(onTurn);
    checkComputerplayer((Player)onTurn);
@@ -306,17 +327,17 @@ void KCubeBoxWidget::readProperties(KConfigBase* config)
 
 /* ***************************************************************** **
 **                               slots                               **
-** ***************************************************************** */ 
+** ***************************************************************** */
 void KCubeBoxWidget::setWaitCursor()
 {
-   setCursor(KCursor::waitCursor());
+   setCursor(Qt::BusyCursor);
 }
 
 
-  
+
 void KCubeBoxWidget::setNormalCursor()
 {
-   setCursor(KCursor::handCursor());
+   setCursor(Qt::PointingHandCursor);
 }
 
 void KCubeBoxWidget::stopHint()
@@ -355,7 +376,7 @@ bool KCubeBoxWidget::checkClick(int row,int column, bool isClick)
 void KCubeBoxWidget::checkComputerplayer(Player player)
 {
    // checking if a process is running or the Widget isn't shown yet
-   if(isActive() || !isVisibleToTLW())  
+   if(isActive() || !isVisible())
       return;
    if((player==One && computerPlOne && currentPlayer==One)
          || (player==Two && computerPlTwo && currentPlayer==Two))
@@ -365,24 +386,24 @@ void KCubeBoxWidget::checkComputerplayer(Player player)
       CubeBox field(*this);
       int row=0,column=0;
       emit startedThinking();
-      bool canceled=!brain.getHint(row,column,(CubeBoxBase<Cube>::Player)player,field);
+      brain.getHint (row, column, (CubeBoxBase<Cube>::Player) player, field);
       emit stoppedThinking();
 
-      if(!canceled)
-      {
-         cubes[row][column]->showHint(500,2);
+      // We do not care if we interrupted the computer.  It was probably taking
+      // too long, so we will just take the best move it had so far.
 
-         bool result=checkClick(row,column,false);
-		 assert(result);
-      }
+      // Blink the cube to be moved (twice).  The realMove = true flag tells
+      // the cube to simulate a mouse click and trigger the move animation,
+      // but not until after the blinking is finished.  The cube's "clicked"
+      // signal is connected to "checkClick (row, column, false)".`
+      cubes[row][column]->showHint (400, 2, true);
    }
-      
 }
 
 /* ***************************************************************** **
 **                         status functions                          **
 ** ***************************************************************** */
- 
+
 bool KCubeBoxWidget::isActive() const
 {
    bool flag=false;
@@ -390,9 +411,9 @@ bool KCubeBoxWidget::isActive() const
       flag=true;
    else if(brain.isActive())
       flag=true;
-   
+
    return flag;
-}  
+}
 
 bool KCubeBoxWidget::isMoving() const
 {
@@ -403,38 +424,51 @@ bool KCubeBoxWidget::isComputer(Player player) const
 {
    if(player==One)
       return computerPlOne;
-   else 
+   else
       return computerPlTwo;
 }
-  
 
 int KCubeBoxWidget::skill() const
 {
    return brain.skill();
 }
-   
-QPalette KCubeBoxWidget::color(Player forWhom)
-{
-   return KCubeWidget::color((KCubeWidget::Owner)forWhom); 
-}
 
 /* ***************************************************************** **
 **                   initializing functions                          **
 ** ***************************************************************** */
 void KCubeBoxWidget::init()
 {
+   setMinimumSize (200, 200);
+   color1 = Prefs::color1();			// Set preferred colors.
+   color2 = Prefs::color2();
+   color0 = Prefs::color0();
+
+   theme.load ("pics/default.desktop");
+   t.start();
+   qDebug() << t.restart() << "msec";
+   svg.load (theme.graphics());
+   qDebug() << t.restart() << "msec" << "SVG loaded ...";
+   if (svg.isValid())
+	qDebug() << "SVG is valid ...";
+   else
+	qDebug() << "SVG is NOT valid ...";
+   drawHairlines = (theme.property("DrawHairlines") == "0") ? false : true;
+
    initCubes();
 
    undoBox=new CubeBox(dim());
-   
+
    currentPlayer=One;
-   moveDelay=100;
+   moveDelay=150;
    moveTimer=new QTimer(this);
    computerPlOne=false;
    computerPlTwo=false;
    KCubeWidget::enableClicks(true);
+
+   // At this point the user's currently preferred number of cubes and colors
+   // are already loaded, so there should be no change and no SVG rendering yet.
    loadSettings();
-   
+
    connect(moveTimer,SIGNAL(timeout()),SLOT(nextLoopStep()));
    connect(this,SIGNAL(startedThinking()),SLOT(setWaitCursor()));
    connect(this,SIGNAL(stoppedThinking()),SLOT(setNormalCursor()));
@@ -446,23 +480,12 @@ void KCubeBoxWidget::init()
 
    emit playerChanged(One);
 }
-   
+
 void KCubeBoxWidget::initCubes()
 {
    const int s=dim();
    int i,j;
-   
-   // create Layout
-   layout=new QGridLayout(this,s,s);
-   
 
-   for(i=0;i<s;i++)
-   {
-      layout->setRowStretch(i,1);
-      layout->setColStretch(i,1);
-   }         
-   
-   
    // create new cubes
    cubes = new KCubeWidget**[s];
    for(i=0;i<s;i++)
@@ -474,20 +497,21 @@ void KCubeBoxWidget::initCubes()
       {
          cubes[i][j]=new KCubeWidget(this);
          cubes[i][j]->setCoordinates(i,j);
-         layout->addWidget(cubes[i][j],i,j);
-         cubes[i][j]->show();
+         cubes[i][j]->setPixmaps (&elements);
          connect(cubes[i][j],SIGNAL(clicked(int,int,bool)),SLOT(stopHint()));
-         connect(cubes[i][j],SIGNAL(clicked(int,int,bool)),SLOT(checkClick(int,int,bool)));
+         connect(cubes[i][j],SIGNAL(clicked(int,int,bool)),
+                             SLOT(checkClick(int,int,bool)));
+         cubes[i][j]->show();
       }
-   
-   // initialize cubes  
+
+   // initialize cubes
    int max=dim()-1;
-      
+
    cubes[0][0]->setMax(2);
    cubes[0][max]->setMax(2);
    cubes[max][0]->setMax(2);
    cubes[max][max]->setMax(2);
-   
+
    for(i=1;i<max;i++)
    {
       cubes[i][0]->setMax(3);
@@ -495,13 +519,155 @@ void KCubeBoxWidget::initCubes()
       cubes[0][i]->setMax(3);
       cubes[max][i]->setMax(3);
    }
-   
+
    for(i=1;i<max;i++)
      for(j=1;j<max;j++)
       {
          cubes[i][j]->setMax(4);
       }
-      
+}
+
+void KCubeBoxWidget::makeStatusPixmaps (const int width)
+{
+   qreal d, p;
+   QImage status (width, width, QImage::Format_ARGB32_Premultiplied);
+   QPainter s (&status);
+   sWidth = width;
+
+   d = width/4.0;
+   p = width/2.0;
+   status.fill (0);
+   svg.render (&s, "player_1");
+   colorImage (status, color1, width);
+   svg.render (&s, "lighting");
+   svg.render (&s, "pip", QRectF (p - d/2.0, p - d/2.0, d, d));
+   status1 = QPixmap::fromImage (status);
+
+   d = width/5.0;
+   p = width/3.0;
+   status.fill (0);
+   svg.render (&s, "player_2");
+   colorImage (status, color2, width);
+   svg.render (&s, "lighting");
+   svg.render (&s, "pip", QRectF (p - d/2.0, p - d/2.0, d, d));
+   svg.render (&s, "pip", QRectF (p + p - d/2.0, p + p - d/2.0, d, d));
+   s.end();
+   status2 = QPixmap::fromImage (status);
+}
+
+void KCubeBoxWidget::makeSVGBackground (const int w, const int h)
+{
+   qDebug() << t.restart() << "msec";
+   QImage img (w, h, QImage::Format_ARGB32_Premultiplied);
+   QPainter p (&img);
+   img.fill (0);
+   svg.render (&p, "background");
+   p.end();
+   background = QPixmap::fromImage (img);
+   qDebug() << t.restart() << "msec" << "SVG background rendered";
+}
+
+void KCubeBoxWidget::makeSVGCubes (const int width)
+{
+   qDebug() << t.restart() << "msec";
+   QImage img (width, width, QImage::Format_ARGB32_Premultiplied);
+   QPainter q;                 // Paints whole faces of the dice.
+
+   QImage pip (width/7, width/7, QImage::Format_ARGB32_Premultiplied);
+   QPainter r;                 // Paints the pips on the faces of the dice.
+
+   elements.clear();
+   for (int i = FirstElement; i <= LastElement; i++) {
+     q.begin(&img);
+     if (i == Pip) {
+       pip.fill (0);
+     }
+     else {
+       img.fill (0);
+     }
+     switch (i) {
+     case Neutral:
+       svg.render (&q, "neutral");
+       colorImage (img, color0, width);
+       svg.render (&q, "lighting");
+       break;
+     case Player1:
+       svg.render (&q, "player_1");
+       colorImage (img, color1, width);
+       svg.render (&q, "lighting");
+       break;
+     case Player2:
+       svg.render (&q, "player_2");
+       colorImage (img, color2, width);
+       svg.render (&q, "lighting");
+       break;
+     case Pip:
+       r.begin(&pip);
+       svg.render (&r, "pip");
+       r.end();
+       break;
+     case BlinkLight:
+       svg.render (&q, "blink_light");
+       break;
+     case BlinkDark:
+       svg.render (&q, "blink_dark");
+       break;
+     default:
+       break;
+     }
+     q.end();
+     elements.append
+       ((i == Pip) ? QPixmap::fromImage (pip) : QPixmap::fromImage (img));
+   }
+   qDebug() << t.restart() << "msec" << "SVG rendered";
+}
+
+void KCubeBoxWidget::colorImage (QImage & img, const QColor & c, const int w)
+{
+   QRgb rgba = c.rgba();
+   for (int i = 0; i < w; i++) {
+      for (int j = 0; j < w; j++) {
+         if (img.pixel (i, j) != 0) {
+	    img.setPixel (i, j, rgba);
+         }
+      }
+   }
+}
+
+void KCubeBoxWidget::paintEvent (QPaintEvent * /* event unused */)
+{
+   QPainter p (this);
+   p.drawPixmap (0, 0, background);
+}
+
+void KCubeBoxWidget::resizeEvent (QResizeEvent * event)
+{
+   qDebug() << endl << "KCubeBoxWidget::resizeEvent:" << event->size() << this->size();
+   reCalculateGraphics (event->size().width(), event->size().height());
+}
+
+void KCubeBoxWidget::reCalculateGraphics (const int w, const int h)
+{
+   int boxSize = (h < w) ? h : w;
+   int frameWidth = boxSize / 30;
+   int hairline = drawHairlines ? frameWidth / 10 : 0;
+   qDebug() << "boxSize" << boxSize << "frameWidth" << frameWidth << "hairline" << hairline;
+   boxSize = boxSize - (2 * frameWidth);
+   cubeSize = ((boxSize - hairline) / dim()) - hairline;
+   boxSize = ((cubeSize + hairline) * dim()) + hairline;
+   topLeft.setX ((w - boxSize)/2);
+   topLeft.setY ((h - boxSize)/2);
+
+   qDebug() << "Dimension:" << dim() << "cubeSize:" << cubeSize << "topLeft:" << topLeft;
+   makeSVGBackground (w, h);
+   makeSVGCubes (cubeSize);
+   for (int i = 0; i < dim(); i++) {
+      for (int j = 0; j < dim(); j++) {
+         cubes[i][j]->move (topLeft.x() + hairline + i * (cubeSize + hairline),
+                            topLeft.y() + hairline + j * (cubeSize + hairline));
+         cubes[i][j]->resize (cubeSize, cubeSize);
+      }
+   }
 }
 
 QSize  KCubeBoxWidget::sizeHint() const
@@ -511,12 +677,9 @@ QSize  KCubeBoxWidget::sizeHint() const
 
 void  KCubeBoxWidget::deleteCubes()
 {
-   if(layout)
-      delete layout;
-     
    CubeBoxBase<KCubeWidget>::deleteCubes();
 }
- 
+
 
 /* ***************************************************************** **
 **                   other private functions                         **
@@ -528,8 +691,12 @@ void KCubeBoxWidget::doMove(int row,int column)
    if(isActive())
       return;
 
-   // for undo-function copy field
-   *undoBox=*this;
+   bool computerMove = ((computerPlOne && currentPlayer==One) ||
+                        (computerPlTwo && currentPlayer==Two));
+   if (! computerMove) { // Make only human-players' moves undoable.
+      // For the undo-function: make a copy of the playfield.
+      *undoBox = *this;
+   }
 
    cubes[row][column]->increase((Cube::Owner)currentPlayer);
 
@@ -539,19 +706,19 @@ void KCubeBoxWidget::doMove(int row,int column)
       startLoop();
    }
    else
-      changePlayer();      
+      changePlayer();
 }
 
 void KCubeBoxWidget::startLoop()
 {
    emit startedMoving();
-   
+
    KCubeWidget::enableClicks(false);
-   
+
    loop.row=0;
    loop.column=0;
    loop.finished=true;
-   
+
    moveTimer->start(moveDelay);
 }
 
@@ -583,7 +750,7 @@ void KCubeBoxWidget::nextLoopStep()
 	    {
 	       stopLoop();
 	       changePlayer();
-		         
+
 	       return;
             }
          }
@@ -594,12 +761,12 @@ void KCubeBoxWidget::nextLoopStep()
          }
       }
    }
-    
- 
+
+
    increaseNeighbours(currentPlayer,loop.row,loop.column);
    cubes[loop.row][loop.column]->decrease();
    loop.finished=false;
-    
+
    if(hasPlayerWon(currentPlayer))
    {
       emit playerWon((int)currentPlayer);
@@ -631,6 +798,10 @@ KCubeBoxWidget::Player KCubeBoxWidget::changePlayer()
    return currentPlayer;
 }
 
+const QPixmap & KCubeBoxWidget::playerPixmap (const int p)
+{
+   return ((p == 1) ? status1 : status2);
+}
 
 void KCubeBoxWidget::increaseNeighbours(KCubeBoxWidget::Player forWhom,int row,int column)
 {
