@@ -19,43 +19,23 @@
 ****************************************/
 
 #include "cardmaps.h"
+
+#include "dealer.h"
 #include "version.h"
 #include "view.h"
-#include "dealer.h"
-#include "ksvgrenderer.h"
-#include "card.h"
-#include "cardcache.h"
-#include "speeds.h"
+
+#include <KCardCache>
+#include <KCardDeckInfo>
+#include <KConfigGroup>
+#include <KDebug>
+#include <KGlobal>
+#include <KSharedConfig>
+
+
 #ifdef __GNUC__
 #warning cardmap should not really require to know the instance!
 #endif
 
-#include <cstdlib>
-#include <cstdio>
-#include <unistd.h>
-#include <cassert>
-
-#include <QPainter>
-#include <QImage>
-#include <QFileInfo>
-#include <QSvgRenderer>
-#include <QPixmapCache>
-#include <QThread>
-#include <QMutex>
-//Added by qt3to4:
-#include <QPixmap>
-#include <QDir>
-
-#include <kconfig.h>
-#include <kdebug.h>
-#include <klocale.h>
-#include <kstandarddirs.h>
-#include <krandom.h>
-#include <carddeckinfo.h>
-#include <kglobalsettings.h>
-#include <kglobal.h>
-#include <kconfiggroup.h>
-
 cardMap *cardMap::_self = 0;
 
 static QHash<AbstractCard::Rank,KCardInfo::Card> ranks;
@@ -64,21 +44,13 @@ static QHash<AbstractCard::Suit,KCardInfo::Suit> suits;
 class cardMapPrivate
 {
 public:
-
     cardMapPrivate()
-    {
-        _wantedCardWidth = 0;
-        m_frontsvg = false;
-        m_backsvg = false;
-        _scale = 1;
-    }
-    double _wantedCardWidth;
-    mutable double _scale;
+      : m_originalCardSize(1, 1),
+        m_currentCardSize(0, 0)
+    {}
     KCardCache m_cache;
-    QSizeF m_backSize;
-    bool m_backsvg;
-    bool m_frontsvg;
-    QRect m_body;
+    QSizeF m_originalCardSize;
+    QSize m_currentCardSize;
 };
 
 cardMap::cardMap() : QObject()
@@ -91,7 +63,7 @@ cardMap::cardMap() : QObject()
     KConfigGroup cs(config, settings_group );
 
     updateTheme(cs);
-    setWantedCardWidth( cs.readEntry( "CardWidth", 100 ) );
+    setCardWidth( cs.readEntry( "CardWidth", 100 ) );
     suits[AbstractCard::Clubs] = KCardInfo::Club;
     suits[AbstractCard::Spades] = KCardInfo::Spade;
     suits[AbstractCard::Diamonds] = KCardInfo::Diamond;
@@ -122,21 +94,13 @@ void cardMap::updateTheme(const KConfigGroup &cs)
     d->m_cache.setFrontTheme( fronttheme );
     d->m_cache.setBackTheme( backtheme );
 
-    d->m_body = QRect();
-
-    d->m_backsvg = CardDeckInfo::isSVGFront( fronttheme );
-    d->m_frontsvg = CardDeckInfo::isSVGBack( backtheme );
-
-    d->m_backSize = d->m_cache.defaultBackSize();
-
-    Q_ASSERT( !d->m_backSize.isNull() );
-    if ( !d->m_backsvg || !d->m_frontsvg )
-        setWantedCardWidth( d->m_backSize.width() );
+    d->m_originalCardSize = d->m_cache.defaultFrontSize( KCardInfo( KCardInfo::Spade, KCardInfo::Ace ) );
+    Q_ASSERT( !d->m_originalCardSize.isNull() );
+    d->m_currentCardSize = d->m_originalCardSize.toSize();
 
     if (PatienceView::instance() && PatienceView::instance()->dscene()) {
+        PatienceView::instance()->dscene()->relayoutPiles();
         PatienceView::instance()->dscene()->rescale(false);
-        if ( d->m_backsvg || d->m_frontsvg )
-            PatienceView::instance()->dscene()->relayoutPiles();
     }
 }
 
@@ -147,54 +111,58 @@ cardMap::~cardMap()
       _self = 0;
 }
 
-double cardMap::wantedCardHeight() const
+QSize cardMap::cardSize() const
 {
-    return d->_wantedCardWidth / d->m_backSize.width() * d->m_backSize.height();
+    return d->m_currentCardSize;
 }
 
-double cardMap::wantedCardWidth() const
+int cardMap::cardWidth() const
 {
-    return d->_wantedCardWidth;
+    return d->m_currentCardSize.width();
+}
+
+int cardMap::cardHeight() const
+{
+    return d->m_currentCardSize.height();
 }
 
 void cardMap::triggerRescale()
 {
-    KSharedConfig::Ptr config = KGlobal::config();
-    KConfigGroup cs(config, settings_group );
-    cs.writeEntry( "CardWidth", d->_wantedCardWidth );
-    config->sync();
-    if (  PatienceView::instance()->dscene() )
+    KConfigGroup cs( KGlobal::config(), settings_group );
+    cs.writeEntry( "CardWidth", d->m_currentCardSize.width() );
+
+    if ( PatienceView::instance() && PatienceView::instance()->dscene() )
          PatienceView::instance()->dscene()->rescale(false);
 }
 
-void cardMap::setWantedCardWidth( double w )
+void cardMap::setCardWidth( int width )
 {
-    if ( w > 200 || w < 10 || d->_wantedCardWidth == w )
+    if ( width > 200 || width < 10 )
         return;
 
-    if ( !d->m_backsvg && !d->m_frontsvg && w != d->m_backSize.width() )
-        return;
+    int height = width * d->m_originalCardSize.height() / d->m_originalCardSize.width();
+    QSize newSize( width, height );
 
-    d->m_body = QRect();
-    kDebug(11111) << "setWantedCardWidth" << w << d->_wantedCardWidth;
-    d->_wantedCardWidth = w;
-    d->_scale = 0;
-    d->m_cache.setSize( QSize( wantedCardWidth(), wantedCardHeight() ) );
-    if (PatienceView::instance())
+    if ( newSize != d->m_currentCardSize )
     {
-       if (PatienceView::instance()->dscene())
-          PatienceView::instance()->dscene()->rescale(false);
-       triggerRescale();
+        d->m_currentCardSize = newSize;
+        d->m_cache.setSize( newSize );
+        triggerRescale();
+
+        QTimer::singleShot( 200, this, SLOT(loadInBackground()) );;
     }
 }
 
 cardMap *cardMap::self() 
 {
-    assert(_self);
+    Q_ASSERT(_self);
     return _self;
 }
 
-QRect cardMap::opaqueRect() const { return d->m_body; }
+QRect cardMap::opaqueRect() const
+{
+    return QRect();
+}
 
 QPixmap cardMap::renderBackside( int variant )
 {
@@ -208,4 +176,9 @@ QPixmap cardMap::renderFrontside( AbstractCard::Rank r, AbstractCard::Suit s )
     return pix;
 }
 
+void cardMap::loadInBackground()
+{
+    d->m_cache.loadTheme( KCardCache::LoadFrontSide | KCardCache::Load52Cards );
+}
+
 #include "cardmaps.moc"
