@@ -11,8 +11,9 @@
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
 */
-#ifndef _DECK_H_
-#define _DECK_H_
+
+#ifndef DECK_H
+#define DECK_H
 
 #include "pile.h"
 
@@ -25,20 +26,20 @@ class Deck: public Pile
 {
 
 private:
-    Deck( DealerScene* parent = 0, int m = 1, int s = 4 );
+    explicit Deck( DealerScene* parent = 0, int m = 1, int s = 4 );
     virtual ~Deck();
 
 public:
-    static void  create_deck( DealerScene *parent = 0, uint m = 1, uint s = 4 );
+    static void create_deck( DealerScene *parent = 0, uint m = 1, uint s = 4 );
+    static void destroy_deck();
     static Deck *deck() { return my_deck; }
 
-    static const long n;
-
     void collectAndShuffle();
 
     Card* nextCard();
 
     uint decksNum() const { return mult; }
+    uint suitsNum() const { return suits; }
     void update();
 
 private: // functions
