@@ -19,12 +19,14 @@
 ****************************************/
 
 #include "grandf.h"
-#include <klocale.h>
-#include <kdebug.h>
+
 #include "deck.h"
-#include <kaction.h>
+#include "speeds.h"
 #include "patsolve/grandf.h"
-#include <assert.h>
+
+#include <KDebug>
+#include <KLocale>
+
 
 Grandf::Grandf( )
     : DealerScene(  )
@@ -43,12 +45,12 @@ Grandf::Grandf( )
 
     for (int i=0; i<7; i++) {
         store[i] = new Pile(5+i, this);
-        store[i]->setPilePos(1+distx*i, 16 );
+        store[i]->setPilePos(1+distx*i, 13);
         store[i]->setAddFlags(Pile::addSpread | Pile::several);
         store[i]->setRemoveFlags(Pile::several | Pile::autoTurnTop);
         store[i]->setObjectName( QString( "store%1" ).arg( i ) ) ;
         store[i]->setCheckIndex(1);
-        store[i]->setReservedSpace( QSizeF( 10, 45 ) );
+        store[i]->setReservedSpace( QSizeF( 10, 40 ) );
     }
 
     setActions(DealerScene::Hint | DealerScene::Demo | DealerScene::Redeal);
@@ -59,31 +61,36 @@ void Grandf::restart() {
     Deck::deck()->collectAndShuffle();
     deal();
     numberOfDeals = 1;
+    emit newCardsPossible( true );
 }
 
-void Grandf::redeal()
+Card *Grandf::newCards()
 {
-    kDebug(11111) << "redeal\n";
-    unmarkAll();
+//     if ( waiting() || demoActive() || isGameWon()  )
+//         return;
 
-    if (numberOfDeals < 3) {
-        collect();
-        deal();
-        numberOfDeals++;
-    }
-    if (numberOfDeals == 3) {
-        emit redealPossible(false);
-    }
-    takeState();
-}
-
-Card *Grandf::demoNewCards()
-{
-    if (numberOfDeals < 3) {
-        redeal();
-        return store[3]->top();
-    } else
+    if (numberOfDeals >= 3)
         return 0;
+
+    unmarkAll();
+    collect();
+    deal();
+    numberOfDeals++;
+
+    Card *t = store[3]->top();
+    Q_ASSERT(t);
+    if (t)
+    {
+       t->turn(false);
+       t->flipTo( t->pos().x(), t->pos().y(), DURATION_FLIP );
+    }
+
+    takeState();
+    considerGameStarted();
+    if (numberOfDeals == 3)
+        emit newCardsPossible(false);
+
+    return t;
 }
 
 void Grandf::deal() {
@@ -122,7 +129,6 @@ void Grandf::deal() {
         if (c)
             c->turn(true);
     }
-    emit redealPossible( true );
 }
 
 /*****************************
@@ -138,13 +144,14 @@ void Grandf::collect() {
 
     for (int pos = 6; pos >= 0; pos--) {
         CardList p = store[pos]->cards();
-        for (CardList::ConstIterator it = p.begin(); it != p.end(); ++it)
+        for (CardList::ConstIterator it = p.constBegin(); it != p.constEnd(); ++it)
             Deck::deck()->add(*it, true);
     }
 }
 
 bool Grandf::checkAdd( int checkIndex, const Pile *c1, const CardList& c2) const {
-    assert (checkIndex == 1);
+    kDebug(11111) << checkIndex << c1->isEmpty();
+    Q_ASSERT (checkIndex == 1);
     if (c1->isEmpty())
         return c2.first()->rank() == Card::King;
     else
@@ -152,7 +159,7 @@ bool Grandf::checkAdd( int checkIndex, const Pile *c1, const CardList& c2) const
                && c2.first()->suit() == c1->top()->suit();
 }
 
-QString Grandf::getGameState() const
+QString Grandf::getGameState()
 {
     return QString::number(numberOfDeals);
 }
@@ -160,14 +167,14 @@ QString Grandf::getGameState() const
 void Grandf::setGameState( const QString &s)
 {
     numberOfDeals = s.toInt();
-    emit redealPossible(numberOfDeals < 3);
+    emit newCardsPossible(numberOfDeals < 3);
 }
 
 static class LocalDealerInfo1 : public DealerInfo
 {
 public:
-    LocalDealerInfo1() : DealerInfo(I18N_NOOP("&Grandfather"), 1) {}
-    virtual DealerScene *createGame() { return new Grandf(); }
+    LocalDealerInfo1() : DealerInfo(I18N_NOOP("Grandfather"), 1) {}
+    virtual DealerScene *createGame() const { return new Grandf(); }
 } gfdi;
 
 #include "grandf.moc"
