@@ -18,12 +18,13 @@
    4 positions, remove lowest card(s) of suit
 */
 
-
 #include "idiot.h"
-#include <klocale.h>
+
 #include "deck.h"
 #include "patsolve/idiot.h"
 
+#include <KLocale>
+
 Idiot::Idiot( )
   : DealerScene( )
 {
@@ -51,7 +52,7 @@ Idiot::Idiot( )
     m_away->setPilePos( 20 + distx * 4, 1);
     m_away->setObjectName( "away" );
 
-    setActions(DealerScene::Hint | DealerScene::Demo);
+    setActions(DealerScene::Hint | DealerScene::Demo | DealerScene::Deal);
     setSolver( new IdiotSolver(this ) );
 }
 
@@ -59,14 +60,19 @@ Idiot::Idiot( )
 void Idiot::restart()
 {
     Deck::deck()->collectAndShuffle();
-    deal();
+
+    // Move the four top cards of the deck to the piles, faceup, spread out.
+    for ( int i = 0; i < 4; ++i )
+        m_play[ i ]->add( Deck::deck()->nextCard(), false );
+
+    emit newCardsPossible(true);
 }
 
 bool Idiot::cardClicked(Card *c)
 {
     // If the deck is clicked, deal 4 more cards.
     if (c->source() == Deck::deck()) {
-        deal();
+        newCards();
         return true;
     }
 
@@ -138,33 +144,40 @@ bool Idiot::cardDblClicked(Card *)
 
 // Deal 4 cards face up - one on each pile.
 //
-
-void Idiot::deal()
-{
-    if ( Deck::deck()->isEmpty() )
-	return;
-
-    // Move the four top cards of the deck to the piles, faceup, spread out.
-    for ( int i = 0; i < 4; i++ )
-	m_play[ i ]->add( Deck::deck()->nextCard(), false );
-}
-
-Card *Idiot::demoNewCards()
+Card *Idiot::newCards()
 {
     if ( Deck::deck()->isEmpty() )
         return 0;
 
-    deal();
+    if ( waiting() )
+        for ( int i = 0; i < 4; ++i )
+            if ( m_play[i]->top() )
+                return m_play[i]->top();
+
+    unmarkAll();
+
+    // Move the four top cards of the deck to the piles, faceup, spread out.
+    for ( int i = 0; i < 4; ++i )
+        m_play[ i ]->add( Deck::deck()->nextCard(), false );
+
+    takeState();
+    considerGameStarted();
+    if ( Deck::deck()->isEmpty() )
+        emit newCardsPossible( false );
 
     return m_play[0]->top();
 }
 
+void Idiot::setGameState(const QString &)
+{
+    emit newCardsPossible( !Deck::deck()->isEmpty() );
+}
 
 static class LocalDealerInfo2 : public DealerInfo
 {
 public:
-    LocalDealerInfo2() : DealerInfo(I18N_NOOP("&Aces Up"), 2) {}
-    virtual DealerScene *createGame() { return new Idiot(); }
+    LocalDealerInfo2() : DealerInfo(I18N_NOOP("Aces Up"), 2) {}
+    virtual DealerScene *createGame() const { return new Idiot(); }
 } ldi4;
 
 
