@@ -18,11 +18,9 @@
 
 ****************************************/
 
-
 #ifndef IDIOT_H
 #define IDIOT_H
 
-
 #include "dealer.h"
 
 
@@ -33,20 +31,19 @@ class Idiot: public DealerScene
     Q_OBJECT
 
 public:
-
     Idiot( );
     virtual bool  isGameWon() const;
+    virtual void restart();
+
+public slots:
+    virtual Card *newCards();
 
 protected:
     virtual bool  cardClicked(Card *);
     virtual bool  cardDblClicked(Card *);
-    virtual Card *demoNewCards();
+
     virtual bool  startAutoDrop()  { return false; }
-
-public slots:
-
-    virtual void restart();
-    void         deal();
+    virtual void  setGameState(const QString &);
 
 private:
     bool canMoveAway(Card *c) const;
