@@ -24,44 +24,55 @@
 
 ****************************************/
 
-#ifndef P_KLONDIKE
-#define P_KLONDIKE
+#ifndef KLONDIKE_H
+#define KLONDIKE_H
 
 #include "dealer.h"
-
 class KlondikePile;
 
+class KSelectAction;
+
+
 class Klondike : public DealerScene {
     friend class KlondikeSolver;
 
     Q_OBJECT
 
 public:
-    explicit Klondike( bool easy );
+    explicit Klondike();
 
-public:
+    virtual void mapOldId(int id);
     virtual void restart();
     virtual bool startAutoDrop();
 
     void redeal(); // put pile back into deck
     void deal();
-    void deal3(); // move up to 3 cards from deck to pile
 
-    virtual bool cardClicked(Card *);
-    virtual void pileClicked(Pile *c);
+    void setEasy( bool easy );
 
-    virtual Card *demoNewCards();
+public slots:
+    virtual Card *newCards();
+
+protected:
+    virtual QString getGameState();
+    virtual QString getGameOptions() const;
+    virtual void setGameOptions(const QString &options);
 
 private:
     bool EasyRules;
     bool redealt;
 
+    KSelectAction *options;
+
     Pile* play[7];
     Pile* target[4];
 
     KlondikePile *pile;
     Card::Rank target_tops[4];
     bool noLongerNeeded(Card::Rank r, Card::Suit s );
+
+private slots:
+    void gameTypeChanged();
 };
 
 class KlondikePile : public Pile
@@ -71,6 +82,7 @@ public:
     int draw() const { return m_draw; }
     virtual QSizeF cardOffset( const Card * ) const;
     virtual void relayoutCards();
+    void setDraws( int _draw );
 
 private:
     int m_draw;
