@@ -1,10 +1,9 @@
-
-#ifndef _GYPSYSOLVER_H
-#define _GYPSYSOLVER_H
-
-#include "patsolve.h"
+#ifndef GYPSYSOLVER_H
+#define GYPSYSOLVER_H
 
 class Gypsy;
+#include "patsolve.h"
+
 
 class GypsySolver : public Solver
 {
@@ -30,5 +29,5 @@ public:
     int deck, outs;
     int params[6];
 };
-#endif
 
+#endif // GYPSYSOLVER_H
